import React from 'react';
import ReactDOM from 'react-dom';
import PropTypes from 'prop-types';
import { withStyles } from '@material-ui/core/styles';
import Select from 'react-select';
import {
  Button,
  Card,
  CardHeader,
  CardMedia,
  CardContent,
  CardActions,
  Checkbox,
  CssBaseline,
  Dialog,
  DialogActions,
  DialogContent,
  DialogContentText,
  DialogTitle,
  Fab,
  FormHelperText,
  FormControl,
  FormControlLabel,
  FormGroup,
  GridList,
  GridListTile,
  Icon,
  Input,
  InputLabel,
  List,
  ListItem,
  ListItemText,
  MenuItem,
  NoSsr,
  Paper,
  Select as MuiSelect, // Changed because of conflicts with react-select
  TextField,
  Typography,
  createMuiTheme,
  colors,
  MuiThemeProvider,
} from '@material-ui/core';

const styles = theme => ({
  button: {
    margin: theme.spacing(1),
  },
  card: {
    minWidth: 200,
    display: 'inline-block',
  },
  divider: {
    height: theme.spacing(2),
  },
  extendedIcon: {
    marginRight: theme.spacing(1),
  },
  fab: {
    margin: theme.spacing(1),
  },
  formControl: {
    margin: theme.spacing(),
    minWidth: 120,
  },
  gridList: {
    width: 500,
    height: 450,

  },
  input: {
    display: 'flex',
    padding: 0,
  },
  leftIcon: {
    marginRight: theme.spacing(1),
  },
  media: {
    height: 0,
    paddingTop: '56.25%', // 16:9
  },
  noOptionsMessage: {
    padding: `${theme.spacing(1)}px ${theme.spacing(2)}px`,
  },
  paper: {
    position: 'absolute',
    zIndex: 1,
    marginTop: theme.spacing(1),
    left: 0,
    right: 0,
  },
  placeholder: {
    position: 'absolute',
    left: 2,
    fontSize: 16,
  },
  pos: {
    marginBottom: 12,
  },
  rightIcon: {
    marginLeft: theme.spacing(1),
  },
  root: {
    flexGrow: 1,
    height: 310,

    // styles added by grid component
    // display: 'flex',
    // flexWrap: 'wrap',
    // justifyContent: 'space-around',
    // overflow: 'hidden',
    // backgroundColor: theme.palette.background.paper,

  },
  selectEmpty: {
    marginTop: theme.spacing(2),
  },
  singleValue: {
    fontSize: 16,
  },
  textField: {
    marginLeft: 8,
    marginRight: 8,
  },
  title: {
    fontSize: 14,
  },
  valueContainer: {
    display: 'flex',
    flexWrap: 'wrap',
    flex: 1,
    alignItems: 'center',
    overflow: 'hidden',
  }
});

class MCF_ElementFactory {
  static createElement(elementConfig) {
    const params = elementConfig.params;
    if (params.parentKey !== undefined)
      params.key = params.parentKey + params.id;
    else
      params.key = params.id;

    switch (elementConfig.type) {
      case 'card':
        return <MCF_Card key={params.key} params={params}/>;
      case 'mediaCard':
        return <MCF_MediaCard key={params.key} params={params}/>;
      case 'form':
        return <MCF_Form key={params.key} params={params}/>;
      case 'button':
        return <MCF_Button key={params.key} params={params}/>;
      case 'formButton':
        return <MCF_FormButton key={params.key} params={params}/>;
      case 'dialogButton':
        return <MCF_DialogButton key={params.key} params={params}/>;
      case 'list':
        return <MCF_List key={params.key} params={params}/>;
      case 'formList':
        return <MCF_FormList key={params.key} params={params}/>;
      case 'list_autocomplete':
        return <MCF_List_Autocomplete key={params.key} params={params}/>;
      case 'formList_autocomplete':
        return <MCF_FormList_Autocomplete key={params.key} params={params}/>;
      case 'text':
        return <MCF_Text key={params.key} params={params}/>;
      case 'formText':
        return <MCF_FormText key={params.key} params={params}/>;
      case 'dialogText':
        return <MCF_DialogText key={params.key} params={params}/>;
      case 'divider':
        return <MCF_Divider key={params.key} params={params}/>;
      case 'fileSelect':
        return <MCF_FileSelect key={params.key} params={params}/>;
      case 'dialogFileSelect':
        return <MCF_DialogFileSelect key={params.key} params={params}/>;
      case 'dialog':
        return <MCF_Dialog key={params.key} params={params}/>;
      case 'dialogDialog':
        return <MCF_DialogDialog key={params.key} params={params}/>;
      case 'selectionControl':
        return <MCF_SelectionControl key={params.key} params={params}/>;
    }
  }
}

class MCF_Card_NoStyled extends React.Component {
  constructor(props) {
    super(props);
    this.params = this.props.params;
  }

  renderContents() {
    return this.params.contents.map(contentConfig => {
      contentConfig.params.parentKey = this.params.key;
      return MCF_ElementFactory.createElement(contentConfig);
    });
  }

  render() {
    const {classes} = this.props;
    return (
      <Card className={classes.card}>
        <CardHeader
          title={this.params.mainText}
          subheader={this.params.secondaryText}
        />
        <CardContent>
          {this.renderContents()}
        </CardContent>
      </Card>
    );
  }
}

class MCF_MediaCard_NoStyled extends MCF_Card_NoStyled {
  constructor(props) {
    super(props);
  }
  render() {
    const {classes} = this.props;
    return (
      <Card className={classes.card}>
        <CardHeader
          title={this.params.mainText}
          subheader={this.params.secondaryText}
        />
        <CardMedia
          className={classes.media}
          image={this.params.mediaUrl}
          title={this.params.title}
        />
        <CardContent>
          {this.renderContents()}
        </CardContent>
      </Card>
    );
  }
}

class MCF_Form_NoStyled extends MCF_Card_NoStyled {
  constructor(props) {
    super(props);
    this.state = {
      params: this.props.params,
    };
    this.params = this.state.params;
    this.params.submitButton.onClickListener = this.submitForm;

    const submitButtonScheme = {
      type: 'button',
      params: this.params.submitButton,
    };
    this.params.contents.push({type: 'divider', params: {}});
    this.params.contents.push(submitButtonScheme);

    this.formInputs = [];
  }

  submitForm = () => {
    this.params.onFormFilled(this.formInputs)
  };

  formUpdater = (newChildParams) => {
    for (let formChild = 0; formChild < this.params.contents.length; formChild++) {
      if (newChildParams.id === this.params.contents[formChild].params.id) {
        this.params.contents[formChild].params = newChildParams;
      }
    }
    this.setState({
      params: this.params,
    });
  };

  renderContents() {
    return this.params.contents.map(contentConfig => {
      contentConfig.params.parentKey = this.params.key;
      contentConfig.params.parentListener = this.formUpdater;
      contentConfig.params.inputContainer = this.formInputs;
      return MCF_ElementFactory.createElement(contentConfig);
    });
  }
}

class MCF_Button_NoStyled extends React.Component {
  constructor(props) {
    super(props);
    this.params = this.props.params;
  }

  render() {
    const {classes} = this.props;
    return (
      <Button
        className={classes.button}
        variant={this.params.variant}
        color={this.params.color}
        disabled={this.params.disabled}
        size={this.params.size}
        value={this.params.value}
        onClick={this.params.onClickListener}
      >
        {this.params.text}
      </Button>
    );
  }
}

class MCF_FormButton_NoStyled extends MCF_Button_NoStyled {
  constructor(props) {
    super(props);
    this.params.onClickListener = this.handleClick;
  }

  handleClick = (event) => {
    this.input = {
      id: this.params.id + '_input',
      value: event.target.value
    };
    let inputDoesntExist = true;
    for (let inputElement = 0; inputElement < this.params.inputContainer.length; inputElement++) {
      if (this.params.inputContainer[inputElement].id === this.input.id) {
        this.params.inputContainer[inputElement] = this.input;
        inputDoesntExist = false;
      }
    }
    if (inputDoesntExist) {
      this.params.inputContainer.push(this.input);
    }
    this.params.value = event.target.value;
    this.params.parentListener(this.params);
  }
}

class MCF_DialogButton_NoStyled extends MCF_Button_NoStyled {
  constructor(props) {
    super(props);
    this.onClickListener = this.params.onClickListener.bind(this);
    this.params.onClickListener = this.handleClick;
  }

  handleClick = () => {
    this.params.parentParams.handleClose();
    this.onClickListener();
  }
}

class MCF_List_NoStyled extends React.Component {
  constructor(props) {
    super(props);
    this.state = {
      params: this.props.params,
    };
    this.params = this.props.params;
  }

  handleInteraction = (event) => {
    this.params.value = event.target.value;
    this.params.onSelectListener({
      id: event.target.name,
      value: event.target.value,
    });
    this.setState({params: this.params});
  };
  createList = () => {
    return (
      this.params.elementsList.map(listElement => {
        return (
          <MenuItem
            key={listElement.name}
            value={listElement.value}
          >
            {listElement.name}
          </MenuItem>
        );
      })
    );
  };

  render() {
    const {classes} = this.props;
    return (
      <FormControl
        className={classes.formControl}
        disabled={this.params.disabled}
        error={this.params.error}
        required={this.params.required}
      >
        <InputLabel
          shrink={this.params.shrink}
          htmlFor={this.params.id}
        >
          {this.params.label}
        </InputLabel>
        <MuiSelect
          value={this.params.value}
          displayEmpty={this.params.displayEmpty}
          autoWidth={this.params.autoWidth}
          onChange={this.handleInteraction}
          input={<Input
            name={this.params.id}
            id={this.params.id}
            readOnly={this.params.readOnly}
          />}
        >
          {this.createList()}
        </MuiSelect>
        <FormHelperText>
          {this.params.helperText}
        </FormHelperText>
      </FormControl>
    );
  }
}

class MCF_FormList_NoStyled extends React.Component {
  constructor(props) {
    super(props);
    this.params = this.props.params;
  }

  handleInteraction = (event) => {
    this.input = {
      id: this.params.id + '_input',
      value: event.target.value
    };
    let inputDoesntExist = true;
    for (let inputElement = 0; inputElement < this.params.inputContainer.length; inputElement++) {
      if (this.params.inputContainer[inputElement].id === this.input.id) {
        this.params.inputContainer[inputElement] = this.input;
        inputDoesntExist = false;
      }
    }
    if (inputDoesntExist) {
      this.params.inputContainer.push(this.input);
    }
    this.params.value = event.target.value;
    this.params.parentListener(this.params);
  };

  createList = () => {
    return (
      this.params.elementsList.map(listElement => {
        return (
          <MenuItem
            key={listElement.name}
            value={listElement.value}
          >
            {listElement.name}
          </MenuItem>
        );
      })
    );
  };

  render() {
    const {classes} = this.props;
    return (
      <FormControl
        className={classes.formControl}
        disabled={this.params.disabled}
        error={this.params.error}
        required={this.params.required}
      >
        <InputLabel
          shrink={this.params.shrink}
          htmlFor={this.params.id}
        >
          {this.params.label}
        </InputLabel>
        <MuiSelect
          value={this.params.value}
          displayEmpty={this.params.displayEmpty}
          autoWidth={this.params.autoWidth}
          onChange={this.handleInteraction}
          input={<Input
            name={this.params.id}
            id={this.params.id}
            readOnly={this.params.readOnly}
          />}
        >
          {this.createList()}
        </MuiSelect>
        <FormHelperText>
          {this.params.helperText}
        </FormHelperText>
      </FormControl>
    );
  }
}

class MCF_Text_NoStyled extends React.Component {
  constructor(props) {
    super(props);
    this.state = {
      params: this.props.params,
    };
    this.params = this.props.params;
  }

  handleInteraction = (event) => {
    this.input = {
      id: this.params.id,
      value: event.target.value
    };
    this.params.value = event.target.value;
    if (this.params.onChangeListener){
      this.params = this.params.onChangeListener(this.params);
    }
    this.setState({params: this.params});
  };

  render() {
    const {classes} = this.props;
    return (
      <form className={classes.container} noValidate autoComplete="off">
        <TextField
          id={this.params.id}
          label={this.params.label}
          className={classes.textField}
          value={this.params.value}
          onChange={this.handleInteraction}
          margin={this.params.margin}
          required={this.params.required}
          error={this.params.error}
          disabled={this.params.disabled}
          placeholder={this.params.placeholder}
          helperText={this.params.helperText}
          fullWidth={this.params.fullWidth}
          variant={this.params.variant}
          type={this.params.type}
          InputLabelProps={{
            shrink: this.params.shrink,
          }}
        />
      </form>
    )
  }
}

class MCF_FormText_NoStyled extends React.Component {
  constructor(props) {
    super(props);
    this.params = this.props.params;
  }

  handleInteraction = (event) => {
    this.input = {
      id: this.params.id + '_input',
      value: event.target.value
    };
    let inputDoesntExist = true;
    for (let inputElement = 0; inputElement < this.params.inputContainer.length; inputElement++) {
      if (this.params.inputContainer[inputElement].id === this.input.id) {
        this.params.inputContainer[inputElement] = this.input;
        inputDoesntExist = false;
      }
    }
    if (inputDoesntExist) {
      this.params.inputContainer.push(this.input);
    }
    this.params.value = event.target.value;
    this.params.parentListener(this.params);
  };

  render() {
    const {classes} = this.props;
    return (
      <form className={classes.container} noValidate autoComplete="off">
        <TextField
          id={this.params.id}
          label={this.params.label}
          className={classes.textField}
          value={this.params.value}
          onChange={this.handleInteraction}
          margin={this.params.margin}
          required={this.params.required}
          error={this.params.error}
          disabled={this.params.disabled}
          placeholder={this.params.placeholder}
          helperText={this.params.helperText}
          fullWidth={this.params.fullWidth}
          variant={this.params.variant}
          type={this.params.type}
          InputLabelProps={{
            shrink: this.params.shrink,
          }}
        />
      </form>
    )
  }
}

class MCF_DialogText_NoStyled extends MCF_Text_NoStyled {
  constructor(props) {
    super(props);
    this.inheritHandleInteraction = this.handleInteraction.bind(this);
    this.handleInteraction = this.extendedHandleInteraction;
  }

  extendedHandleInteraction = (event) => {
    this.inheritHandleInteraction(event);
    this.params.parentParams.handleClose();
  };
}

function MCF_List_Autocomplete_NoOptionsMessage(props) {
  return (
    <Typography
      color="textSecondary"
      className={props.selectProps.classes.noOptionsMessage}
      {...props.innerProps}
    >
      {props.children}
    </Typography>
  );
}

function MCF_List_Autocomplete_inputComponent({inputRef, ...props}) {
  return <div ref={inputRef} {...props} />;
}

function MCF_List_Autocomplete_Control(props) {
  return (
    <TextField
      fullWidth
      InputProps={{
        inputComponent: MCF_List_Autocomplete_inputComponent,
        inputProps: {
          className: props.selectProps.classes.input,
          inputRef: props.innerRef,
          children: props.children,
          ...props.innerProps,
        },
      }}
      {...props.selectProps.textFieldProps}
    />
  );
}

function MCF_List_Autocomplete_Option(props) {
  return (
    <MenuItem
      buttonRef={props.innerRef}
      selected={props.isFocused}
      component="div"
      style={{
        fontWeight: props.isSelected ? 500 : 400,
      }}
      {...props.innerProps}
    >
      {props.children}
    </MenuItem>
  );
}

function MCF_List_Autocomplete_Placeholder(props) {
  return (
    <Typography
      color="textSecondary"
      className={props.selectProps.classes.placeholder}
      {...props.innerProps}
    >
      {props.children}
    </Typography>
  );
}

function MCF_List_Autocomplete_SingleValue(props) {
  return (
    <Typography className={props.selectProps.classes.singleValue} {...props.innerProps}>
      {props.children}
    </Typography>
  );
}

function MCF_List_Autocomplete_ValueContainer(props) {
  return <div className={props.selectProps.classes.valueContainer}>{props.children}</div>;
}

function MCF_List_Autocomplete_Menu(props) {
  return (
    <Paper square className={props.selectProps.classes.paper} {...props.innerProps}>
      {props.children}
    </Paper>
  );
}

const MCF_List_Autocomplete_components = {
  Control: MCF_List_Autocomplete_Control,
  Menu: MCF_List_Autocomplete_Menu,
  NoOptionsMessage: MCF_List_Autocomplete_NoOptionsMessage,
  Option: MCF_List_Autocomplete_Option,
  Placeholder: MCF_List_Autocomplete_Placeholder,
  SingleValue: MCF_List_Autocomplete_SingleValue,
  ValueContainer: MCF_List_Autocomplete_ValueContainer,
};

class MCF_List_Autocomplete_NoStyled extends React.Component {
  constructor(props) {
    super(props);
    this.params = this.props.params;
  }

  state = {
    single: null,
  };

  createOptionsList = () => {
    return this.params.options.map(suggestion => ({
      value: suggestion,
      label: suggestion,
    }));
  };

  render() {
    const {classes, theme} = this.props;
    const selectStyles = {
      input: base => ({
        ...base,
        color: theme.palette.text.primary,
        '& input': {
          font: 'inherit',
        },
      }),
    };
    return (
      <div className={classes.root}>
        <NoSsr>
          <Select
            classes={classes}
            styles={selectStyles}
            options={this.createOptionsList()}
            components={MCF_List_Autocomplete_components}
            value={this.state.value}
            onChange={this.params.onSelectListener}
            placeholder={this.params.placeholder}
            isClearable={this.params.isClearable}
          />
          <div className={classes.divider}/>
          <FormHelperText>
            {this.params.helperText}
          </FormHelperText>
        </NoSsr>
      </div>
    );
  }
}

class MCF_FormList_Autocomplete_NoStyled extends React.Component {
  constructor(props) {
    super(props);
    this.params = this.props.params;
  }

  state = {
    single: null,
  };

  handleChange = (option) => {
    this.setState({
      single: option,
    });
    this.input = {
      id: this.params.id + '_input',
      value: option.value
    };
    let inputDoesntExist = true;
    for (let inputElement = 0; inputElement < this.params.inputContainer.length; inputElement++) {
      if (this.params.inputContainer[inputElement].id === this.input.id) {
        this.params.inputContainer[inputElement] = this.input;
        inputDoesntExist = false;
      }
    }
    if (inputDoesntExist) {
      this.params.inputContainer.push(this.input);
    }
    this.params.value = option.value;
    this.params.parentListener(this.params);
  };
  createOptionsList = () => {
    return this.params.options.map(suggestion => ({
      value: suggestion,
      label: suggestion,
    }));
  };

  render() {
    const {classes, theme} = this.props;
    const selectStyles = {
      input: base => ({
        ...base,
        color: theme.palette.text.primary,
        '& input': {
          font: 'inherit',
        },
      }),
    };
    return (
      <div className={classes.root}>
        <NoSsr>
          <Select
            classes={classes}
            styles={selectStyles}
            options={this.createOptionsList()}
            components={MCF_List_Autocomplete_components}
            value={this.state.value}
            onChange={this.handleChange}
            placeholder={this.params.placeholder}
            isClearable={this.params.isClearable}
          />
          <div className={classes.divider}/>
          <FormHelperText>
            {this.params.helperText}
          </FormHelperText>
        </NoSsr>
      </div>
    );
  }
}

class MCF_Divider_NoStyled extends React.Component {
  constructor(props) {
    super(props);
    this.params = this.props.params;
  }

  render() {
    const {classes, theme} = this.props;
    return (
      <div className={classes.divider}/>
    );
  }
}

class MCF_FileSelect_NoStyled extends React.Component {
  constructor(props) {
    super(props);
    this.params = this.props.params;
  }
  handleUploadSuccess(){
    this.params.onUpload(this.params);
  }
  componentDidMount(){
    this.form = document.getElementById(this.params.id);

    this.form.addEventListener('submit', e => {
      e.preventDefault();

      const files = document.querySelector('[type=file]').files;
      this.params.files = files;
      if (files.length > 0){
        const formData = new FormData();

        for (let i = 0; i < files.length; i++) {
          let file = files[i];

          formData.append('files[]', file);
        }

        fetch(this.params.url, {
          method: 'POST',
          body: formData
        }).then(response => {
          this.handleUploadSuccess(this.params.files);
        });
      }
    });
  }

  render() {
    if (this.params.acceptedFormats === undefined){
      this.params.acceptedFormats = '*/*';
    }
    const {classes} = this.props;
    return (
      <form id={this.params.id} method="post" encType="multipart/form-data">
        <input type="file" name="files[]" multiple accept={this.params.acceptedFormats}></input>
        <input type="submit" value="Upload File" name="submit"></input>
      </form>
    )
  }
}

class MCF_DialogFileSelect_NoStyled extends MCF_FileSelect_NoStyled {
  constructor(props){
    super(props);
  }
  handleUploadSuccess(){
    this.params.parentParams.handleClose();
    this.params.onUpload(this.params);
  }
}

class MCF_Dialog_NoStyled extends React.Component {
  constructor(props) {
    super(props);
    this.params = this.props.params;
    this.state = {open: this.params.open};
  }

  handleClickOpen = () => {
    this.setState({open: true});
    if (this.params.openListener) {
      this.params.openListener();
    }
  };
  handleClose = () => {
    this.setState({open: false});
    if (this.params.closeListener) {
      this.params.closeListener();
    }
  };

  renderContents() {
    return this.params.contents.map(contentConfig => {
      contentConfig.params.parentParams = this.params;
      contentConfig.params.parentParams.handleClose = this.handleClose;
      contentConfig.params.parentParams.handleClickOpen = this.handleClickOpen;
      contentConfig.params.parentKey = this.params.key;
      return MCF_ElementFactory.createElement(contentConfig);
    });
  }

  renderActionContents() {
    return this.params.actionContents.map(contentConfig => {
      contentConfig.params.parentParams = this.params;
      contentConfig.params.parentParams.handleClose = this.handleClose;
      contentConfig.params.parentParams.handleClickOpen = this.handleClickOpen;
      contentConfig.params.parentKey = this.params.key;
      return MCF_ElementFactory.createElement(contentConfig);
    });
  }

  render() {
    const {classes} = this.props;

    return (
      <div>
        <Button
          variant={this.params.openButtonVariant}
          color={this.params.openButtonColor}
          onClick={this.handleClickOpen}>
          {this.params.openButtonText}
        </Button>
        <Dialog
          open={this.state.open}
          onClose={this.handleClose}
          aria-labelledby={this.params.id}
        >
          <DialogTitle id={this.params.id}>
            {this.params.titleText}
          </DialogTitle>
          <DialogContent>
            <DialogContentText>
              {this.params.contentText}
            </DialogContentText>
            {this.renderContents()}
          </DialogContent>
          <DialogActions>
            {this.renderActionContents()}
          </DialogActions>
        </Dialog>
      </div>
    );
  }
}

class MCF_DialogDialog_NoStyled extends MCF_Dialog_NoStyled {
  constructor(props){
    super(props);
  }
  handleClickOpen = () => {
    this.setState({open: true});
    if (typeof(this.params.openListener) !== undefined) {
      this.params.openListener(this.params);
    }
  };
}

class MCF_SelectionControl_NoStyled extends React.Component {
  state = {};
  constructor(props) {
    super(props);
    this.params = this.props.params;
    this.params.checkboxes.map(checkbox => {
      this.state[checkbox.name] = checkbox.isChecked;
    })
  }

  handleChange = name => event => {
    this.params.onChangeListener({
      id: name,
      value: event.target.checked
    });
    this.setState({[name]: event.target.checked});
  };
  createCheckboxes = () => {
    return (
      this.params.checkboxes.map(checkbox => {
        return (
          <FormControlLabel
            id={checkbox.name}
            key={checkbox.name}
            control={
              <Checkbox
                checked={this.state[checkbox.name]}
                onChange={this.handleChange(checkbox.name)}
                value={checkbox.name}
              />
            }
            label={checkbox.label}
          />
        );
      })
    );
  };

  render() {
    const {classes} = this.props;
    return (
      <FormGroup
        id={this.params.id}
        row={this.params.isRow}
      >
        {this.createCheckboxes()}
      </FormGroup>
    )
  }
}

MCF_Card_NoStyled.propTypes = {classes: PropTypes.object.isRequired};
MCF_MediaCard_NoStyled.propTypes = {classes: PropTypes.object.isRequired};
MCF_Form_NoStyled.propTypes = {classes: PropTypes.object.isRequired};
MCF_Button_NoStyled.propTypes = {classes: PropTypes.object.isRequired};
MCF_FormButton_NoStyled.propTypes = {classes: PropTypes.object.isRequired};
MCF_DialogButton_NoStyled.propTypes = {classes: PropTypes.object.isRequired};
MCF_List_NoStyled.propTypes = {classes: PropTypes.object.isRequired};
MCF_FormList_NoStyled.propTypes = {classes: PropTypes.object.isRequired};
MCF_List_Autocomplete_NoStyled.propTypes = {classes: PropTypes.object.isRequired};
MCF_FormList_Autocomplete_NoStyled.propTypes = {classes: PropTypes.object.isRequired};
MCF_Text_NoStyled.propTypes = {classes: PropTypes.object.isRequired};
MCF_FormText_NoStyled.propTypes = {classes: PropTypes.object.isRequired};
MCF_DialogText_NoStyled.propTypes = {classes: PropTypes.object.isRequired};
MCF_Divider_NoStyled.propTypes = {classes: PropTypes.object.isRequired};
MCF_FileSelect_NoStyled.propTypes = {classes: PropTypes.object.isRequired};
MCF_DialogFileSelect_NoStyled.propTypes = {classes: PropTypes.object.isRequired};
MCF_Dialog_NoStyled.propTypes = {classes: PropTypes.object.isRequired,};
MCF_DialogDialog_NoStyled.propTypes = {classes: PropTypes.object.isRequired,};
MCF_SelectionControl_NoStyled.propTypes = {classes: PropTypes.object.isRequired};
const MCF_Card = withStyles(styles)(MCF_Card_NoStyled);
const MCF_MediaCard = withStyles(styles)(MCF_MediaCard_NoStyled);
const MCF_Form = withStyles(styles)(MCF_Form_NoStyled);
const MCF_Button = withStyles(styles)(MCF_Button_NoStyled);
const MCF_FormButton = withStyles(styles)(MCF_FormButton_NoStyled);
const MCF_DialogButton = withStyles(styles)(MCF_DialogButton_NoStyled);
const MCF_List = withStyles(styles)(MCF_List_NoStyled);
const MCF_FormList = withStyles(styles)(MCF_FormList_NoStyled);
const MCF_List_Autocomplete = withStyles(styles, {withTheme: true})(MCF_List_Autocomplete_NoStyled);
const MCF_FormList_Autocomplete = withStyles(styles, {withTheme: true})(MCF_FormList_Autocomplete_NoStyled);
const MCF_Text = withStyles(styles)(MCF_Text_NoStyled);
const MCF_FormText = withStyles(styles)(MCF_FormText_NoStyled);
const MCF_DialogText = withStyles(styles)(MCF_DialogText_NoStyled);
const MCF_Divider = withStyles(styles)(MCF_Divider_NoStyled);
const MCF_FileSelect = withStyles(styles)(MCF_FileSelect_NoStyled);
const MCF_DialogFileSelect = withStyles(styles)(MCF_DialogFileSelect_NoStyled);
const MCF_Dialog = withStyles(styles)(MCF_Dialog_NoStyled);
const MCF_DialogDialog = withStyles(styles)(MCF_DialogDialog_NoStyled);
const MCF_SelectionControl = withStyles(styles)(MCF_SelectionControl_NoStyled);

function MCF_render(schemeToRender, referenceSelector) {
  ReactDOM.render(
    MCF_ElementFactory.createElement(schemeToRender),
    document.querySelectorAll(referenceSelector)[0]
  );
}

export {MCF_render as render};
